@@ -20,5 +20,11 @@ def log(message,style=None):
             print(_Fore.RED + message + _Style.RESET_ALL)
         if style == "success":
             print(_Fore.GREEN + message + _Style.RESET_ALL)
+        if style == "blue":
+            print(_Fore.CYAN + message + _Style.RESET_ALL)
+        if style == "magenta":
+            print(_Fore.MAGENTA + message + _Style.RESET_ALL)
+        if style == "yellow":
+            print(_Fore.YELLOW + message + _Style.RESET_ALL)
         
 
