@@ -354,7 +354,7 @@ class TableDataManager:
             self.db.insert_data[self.insert_type][self.name] = []
         return self.db.insert_data[self.insert_type][self.name]
 
-    def set_insert_data(self,data,save_all=False):
+    def set_insert_data(self,data,save_all=False,**kwargs):
         '''
             Set the insert data for the table.
 
@@ -385,8 +385,11 @@ class TableDataManager:
             `method_name`: insert_data
             * @xxx [06-02-2022 09:45:42]: documentation for insert_data
         '''
+        overwrite = _obj.get_kwarg(['overwrite'],False,bool,**kwargs)
 
-
+        if overwrite is True:
+            self.db.insert_data[self.insert_type][self.name] = []
+            # self.save(True)
         # @Mstep [IF] if data is None
         if data is None:
             # @Mstep [] set the insert data to an empty list.
@@ -404,7 +407,7 @@ class TableDataManager:
         rows = []
         # @Mstep [LOOP] iterate the rows in the data provided.
         for row in data:
-            # print(f"d: {d}")
+            # print(f"row: {row}")
             # @Mstep [LOOP] iterate the columns of the table.
             for col_name in self.table.column_names:
                 # @Mstep [IF] if the column is the primary key.
@@ -428,6 +431,8 @@ class TableDataManager:
             if has_required_columns(self.table,row,True):
                 # @Mstep [] append the row to the rows list.
                 rows.append(row)
+            # else:
+            #     print(f"missing columns: {row}")
 
         # @Mstep [] if the rows list contains at least one row.
         if len(rows) > 0:
@@ -648,6 +653,7 @@ def has_required_columns(table,row,print_errors=True):
                 if print_errors:
                     msg = f"{k} is a required column in {table.name}, None was provided."
                     alerts.append(msg)
+                    _config.log(msg,"error")
                     # print(f"{k} is a required column in {table.name}, None was provided.")
     if len(alerts) > 0:
         logged = []
