@@ -12,13 +12,17 @@
     `created`: 06-03-2022 10:22:15
     `memberOf`: dict_utils
 '''
-# import random
 
+# import re
+from types import SimpleNamespace
 from typing import Union as _Union
-from typing import Iterable as _Iterable
+# from typing import Iterable as _Iterable
+from typing import TypeVar as _TypeVar
 from colorama import Fore as _Fore
 from colorama import Style as _Style
 import utils.list_utils as _lu
+import utils.string_utils as _csu
+import collections
 
 def set_defaults(default_vals, obj,**kwargs):
     '''
@@ -98,6 +102,34 @@ def keys_to_lower(dictionary):
     '''
     return {k.lower(): v for k, v in dictionary.items()}
 
+def keys_to_snake_case(data:dict)->dict:
+    '''
+        Convert all keys in the dictionary to snake_case
+
+        ----------
+
+        Arguments
+        -------------------------
+        `data` {dict}
+            The dictionary to format.
+
+        Return {dict}
+        ----------------------
+        The dictionary with snake case keys.
+
+        Meta
+        ----------
+        `author`: Colemen Atwood
+        `created`: 06-11-2022 08:53:09
+        `memberOf`: dict_utils
+        `version`: 1.0
+        `method_name`: keys_to_snake_case
+        * @xxx [06-11-2022 08:54:03]: documentation for keys_to_snake_case
+    '''
+
+
+    return {_csu.to_snake_case(k): v for k, v in data.items()}
+
 def keys_to_list(data:dict)->list:
     '''
         return all keys in a dictionary as a list.
@@ -448,7 +480,51 @@ def strip_nulls(data:dict)->dict:
             new_data[k] = v
     return new_data
 
-def longest_key(data:dict)->_Iterable[int,str]:
+def replace_key(data:dict,find:str,replace:str,**kwargs)->dict:
+    '''
+        Find and rename a key in a dictionary.
+
+        ----------
+
+        Arguments
+        -------------------------
+        `data` {dict}
+            The dictionary to search within.
+
+        `find` {str}
+            The key to replace.
+
+        `replace` {str}
+            The new key to rename `find` to.
+
+        Keyword Arguments
+        -------------------------
+        [`case`=True] {bool}
+            if False, casing is ignored.
+
+        Return {dict}
+        ----------------------
+        The formatted dict with the find key renamed.
+
+        If the find key is not found, the dictionary is unaltered.
+
+        Meta
+        ----------
+        `author`: Colemen Atwood
+        `created`: 06-11-2022 08:42:18
+        `memberOf`: dict_utils
+        `version`: 1.0
+        `method_name`: replace_key
+        * @xxx [06-11-2022 08:53:03]: documentation for replace_key
+    '''
+    case_sensitive = get_kwarg(['case','case_sensitive'],True,(bool),**kwargs)
+
+    if case_sensitive is True:
+        return {replace if k == find else k:v for k,v in data.items()}
+    else:
+        return {replace if k.lower() == find.lower() else k:v for k,v in data.items()}
+
+def longest_key(data:dict)->tuple:
     '''
         Get the longest key in a dictionary.
 
@@ -488,3 +564,175 @@ def longest_key(data:dict)->_Iterable[int,str]:
                 longest_len = val_len
                 longest_val = val
     return (longest_len,longest_val)
+
+def reorder_keys(data:dict,**kwargs)->dict:
+    '''
+        Reorder the keys in a dictionary.
+
+        If a `keys` list/dict is provided it will sort the `data` keys to match it.
+        Any keys are exit in `data` but NOT in `keys` will be appended to the end
+        in alphabetical order.
+        
+        If `alpha` is True, it will sort the dictionary alphabetically.
+
+        ----------
+
+        Arguments
+        -------------------------
+        `data` {dict}
+            The dictionary to reorder.
+
+
+        Keyword Arguments
+        -------------------------
+        [`keys`=None] {list|dict}
+            The list of keys or a dict to match the order of.
+
+        [`alpha`=False] {bool}
+            If True, the dictionary's keys will be sorted alphabetically.
+            This is also the default if the `keys` arg is not provided.
+
+        [`case`=False] {bool}
+            Only applies if `alpha` is True
+            If True, keys that start with lower case letters will be sorted first.
+
+        Return {type}
+        ----------------------
+        return_description
+
+        Meta
+        ----------
+        `author`: Colemen Atwood
+        `created`: 06-11-2022 09:21:30
+        `memberOf`: dict_utils
+        `version`: 1.0
+        `method_name`: reorder_keys
+        * @TODO []: documentation for reorder_keys
+    '''
+    # keys:_Union[list,dict]
+    keys = get_kwarg(['keys'],None,(dict,list,tuple),**kwargs)
+    alphabetically = _lu.force_list(get_kwarg(['alpha'],False,(bool),**kwargs))
+    case_sensitive = _lu.force_list(get_kwarg(['case','case_sensitive'],False,(bool),**kwargs))
+
+    if keys is None:
+        if alphabetically:
+            # @Mstep [] get all keys from the dictionary
+            dkeys = list(data.keys())
+            if case_sensitive is False:
+                # @Mstep [] sort the keys alphabetically.
+                keys = sorted(dkeys, key=str.lower)
+            if case_sensitive is True:
+                keys = sorted(dkeys)
+
+
+
+    if keys is not None:
+        if isinstance(keys,(dict)):
+            keys = list(keys.keys())
+
+        new_data = {}
+        # @Mstep [] find keys that exist in the data but not in the keys list.
+        missing_keys = _lu.find_list_diff(keys_to_list(data),keys)
+        if len(missing_keys) > 0:
+            missing_keys = sorted(missing_keys, key=str.lower)
+
+        # @Mstep [loop] iterate the keys list.
+        for k in keys:
+            # @Mstep [if] if the key exists in the data dictionary
+            if k in data:
+                # @Mstep [] add the key to the new_data dict
+                new_data[k] = data[k]
+
+        # @Mstep [LOOP] iterate the missing keys list.
+        for m in missing_keys:
+            # @Mstep [] add the missing keys to the end of the new_data dict
+            new_data[m] = data[m]
+
+        return new_data
+
+def find_dict_key_diff(dict_one:dict,dict_two:dict,**kwargs)->list:
+    '''
+        Find keys in dict_one do that do not exist in dict_two
+
+        ----------
+
+        Arguments
+        -------------------------
+        `dict_one` {dict}
+            The first dict, the primary list for comparison
+        `dict_two` {dict}
+            The dict to find missing keys in.
+
+        Keyword Arguments
+        -------------------------
+        [`reverse`=False] {bool}
+            If True, Find keys in dict_two do that do not exist in dict_one
+
+        Return {list}
+        ----------------------
+        A list of keys that exist in dict_one but not dict_two
+
+        if reverse is True,\n
+        A list of keys that exist in dict_two but not dict_one
+
+
+        Meta
+        ----------
+        `author`: Colemen Atwood
+        `created`: 06-11-2022 09:16:58
+        `memberOf`: dict_utils
+        `version`: 1.0
+        `method_name`: find_dict_key_diff
+        * @xxx [06-11-2022 09:20:49]: documentation for find_dict_key_diff
+    '''
+    reverse = get_kwarg(['reverse'],False,(bool),**kwargs)
+    if reverse is False:
+        return _lu.find_list_diff(list(dict_one.keys()),list(dict_two.keys()))
+    else:
+        return _lu.find_list_diff(list(dict_two.keys()),list(dict_one.keys()))
+
+
+def flatten(d, parent_key='', sep='.'):
+    '''
+        Flatten a dictionary with dot_notation.
+
+        ----------
+
+        Arguments
+        -------------------------
+        `data` {dict}
+            The dictionary to flatten.
+
+        Return {dict}
+        ----------------------
+        The flattened dictionary.
+
+        Meta
+        ----------
+        `author`: Colemen Atwood
+        `created`: 06-14-2022 13:16:43
+        `memberOf`: dict_utils
+        `version`: 1.0
+        `method_name`: flatten
+        * @xxx [06-14-2022 13:17:42]: documentation for flatten
+    '''
+
+
+    items = []
+    for k, v in d.items():
+        new_key = parent_key + sep + k if parent_key else k
+        if isinstance(v, collections.MutableMapping):
+            items.extend(flatten(v, new_key, sep=sep).items())
+        else:
+            items.append((new_key, v))
+    return dict(items)
+
+class NestedNamespace(SimpleNamespace):
+    def __init__(self, dictionary, **kwargs):
+        # _server_type = _TypeVar('_server_type', bound=self)
+        super().__init__(**kwargs)
+        for key, value in dictionary.items():
+            if isinstance(value, dict):
+                self.__setattr__(key, NestedNamespace(value))
+            else:
+                self.__setattr__(key, value)
