@@ -130,7 +130,44 @@ def sql_type_to_python(value):
     return None
 
 def parse_column_data(value=None):
-
+    
+    # @Mstep [] capture the default value if it is found in the line.
+    default_value = None
+    match = _re.findall(r"(\s+DEFAULT\s+([\'\"][^\']*[\'\"]|[0-9\.]))",value,_re.IGNORECASE)
+    if match is not None and len(match) > 0:
+        match = list(match[0])
+        # print(f"default found: {match}")
+        default_value = match[1]
+        if _re.match(r"^[0-9]*$",match[1]) is not None:
+            default_value = int(match[1])
+        # if _re.match(r"^[0-9\.]+$",match[1]) is not None:
+        #     default_value = float(match[1])
+        value = value.replace(match[0],"")
+    
+    # print(f"value: {value}")
+    
+    
+    # @Mstep [] capture the default value if it is found in the line.
+    comment_value = None
+    match = _re.findall(r"(\s+COMMENT\s+([\'\"][^\']*[\'\"]|[0-9\.]))",value,_re.IGNORECASE)
+    if match is not None and len(match) > 0:
+        match = list(match[0])
+        # print(f"default found: {match}")
+        comment_value = str(match[1])
+        comment_value = _csu.strip(comment_value,["'",'"'])
+        # comment_value = _csu.escape_quoted_chars(comment_value,True)
+        # json = _csu.parse.safe_load_json(comment_value)
+        # if json is not False:
+        #     comment_value = json
+        #     print(f"comment_value: {comment_value}")
+        value = value.replace(match[0],"")
+        
+        
+        
+        
+    # print(f"value: {value}")
+    
+    
     # value = "`timestamp`       int NULL DEFAULT NULL COMMENT 'Timestamp of when the task was created.' ,"
     _tick,_double_quote,_single_quote,_comma = [Suppress(x) for x in list('`"\',')]
     _quote = (_tick|_double_quote|_single_quote)
@@ -149,23 +186,26 @@ def parse_column_data(value=None):
     null_vals = CaselessKeyword("NULL") | CaselessKeyword("NOT NULL")
     null_type = null_vals.setResultsName('null_type')
 
-    def_k = Suppress(CaselessKeyword('DEFAULT'))
-    default = Optional(def_k + Word(alphanums))
-    quote = QuotedString("'", esc_quote="''",unquote_results=False).search_string(value)
-    if len(quote) > 0:
-        quote = quote[0][0]
-        default = Optional(default|def_k + quote).setResultsName('default')
+    # def_k = Suppress(CaselessKeyword('DEFAULT'))
+    # default = Optional(def_k + Word(alphanums))
+    # quote = QuotedString("'", esc_quote="''",unquote_results=False).search_string(value)
+    # if len(quote) > 0:
+    #     # print(f"parse_column_data. quote found in column. {quote}")
+    #     quote = quote[0][0]
+    #     default = Optional(default|def_k + quote).setResultsName('default')
+    #     # print(f"    default:{default}")
 
-    com_k = Suppress(CaselessKeyword('COMMENT'))
-    comment = Optional(com_k + Word(alphanums))
-    quote = QuotedString("'", esc_quote="''",unquote_results=False).search_string(value)
-    if len(quote) > 0:
-        quote = quote[0][0]
-        comment = Optional(com_k + quote).setResultsName('comment')
+    # com_k = Suppress(CaselessKeyword('COMMENT'))
+    # comment = Optional(com_k + Word(alphanums))
+    # quote = QuotedString("'", esc_quote="''",unquote_results=False).search_string(value)
+    # if len(quote) > 0:
+    #     quote = quote[0][0]
+    #     comment = Optional(com_k + quote).setResultsName('comment')
 
 
 
-    grammar = column_name + column_type + null_type + default + comment
+    # grammar = column_name + column_type + null_type + default + comment
+    grammar = column_name + column_type + null_type
     res = ZeroOrMore(Group(grammar).setResultsName('data')).parseString(value)
 
     output = None
@@ -177,8 +217,8 @@ def parse_column_data(value=None):
             "name":None,
             "type":None,
             "allow_nulls":None,
-            "default":None,
-            "comment":None,
+            "default":default_value,
+            "comment":comment_value,
         }
         new_data = {}
         for k,v in data.items():
@@ -1406,7 +1446,11 @@ def parse(sql):
         if 'columns' in tb:
             for col in tb['columns']:
                 if col['comment'] is not None:
-                    col['comment'] = _sql.escape_quoted_chars(col['comment'],True)
+                    if isinstance(col['comment'],(str)):
+                        col['comment'] = _sql.escape_quoted_chars(col['comment'],True)
+                        json = _csu.parse.safe_load_json(col['comment'])
+                        if json:
+                            col['comment'] = json
     return data
 
 
